#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-06-28
# File  : snmp_configuration.py

#
# Monitor snmp version used by CMK for a host object
#
# 2023-06-28: initial release
# 2023-07-09: added WATO parameters
# 2023-07-11: added check key length, key complexity,  use of default keys
# 2023-08-04: renamed from snmp_version to snmp_configuration
#             streamlined function parameters
#             added option to check for reversed keys in default keys list
#             added report only option
#             added top level _check_key function
#             added check for duplicate char in key

# ToDo
#  maybe add number of repeated chars?


import re
from typing import List
from cmk.base.check_api import host_name
from cmk.base.config import get_config_cache
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    SNMPTree,
    Result,
    State,
    exists,
)


def _check_expected(value, message: str, state: int, report_only: bool, excepted: List):
    if report_only:
        state = 0

    if value in excepted:
        yield Result(state=State.OK, summary=f'{message}: {value}')
    else:
        yield Result(state=State(state), summary=f'{message} {value}')


def _result(message: str, state: int, report_only: bool):
    if report_only:
        yield Result(state=State.OK, summary=message)
    else:
        yield Result(state=State(state), notice=message)


def _check_key_length(key: str, message: str, state: int, report_only: bool, min_length: int):
    if len(key) < min_length:
        yield from _result(
            state=state, report_only=report_only, message=f'{message} length below {min_length}'
        )
    else:
        yield Result(state=State.OK, notice=f'{message} length is not below {min_length}')


def _check_default_key(
        key: str, message: str, state: int, report_only: bool, default_keys: List[str], reversed_key: bool
):
    if key.lower() in default_keys:
        yield from _result(state=state, report_only=report_only, message=f'{message} uses default value')
    elif reversed_key and key.lower()[::-1] in default_keys:
        yield from _result(state=state, report_only=report_only, message=f'{message} uses default value (reverse)')
    else:
        yield Result(state=State.OK, notice=f'{message} is not using default keys')


def _check_key_complexity(key: str, message: str, state: int, report_only: bool, regex: List[str]):
    _regex = ''.join(regex)

    if not re.search(_regex, key):
        yield from _result(state=state, report_only=report_only, message=f'{message} complexity is not met')
    else:
        yield Result(state=State.OK, notice=f'{message} complexity is met.')


def _check_repeated_chars(key: str, message: str, state: int, report_only: bool):
    _regex = '(.)\\1{1}'  # {x} number of repetitions
    if re.search(_regex, key):
        # re.search(_regex, key).group() gives the repeated chars
        yield from _result(
            state=state, report_only=report_only,
            message=f'{message} has repeated chars'
        )
    else:
        yield Result(state=State.OK, notice=f'{message} has no repeated chars')


def _check_key(
        key: str,
        message: str,
        report_only: bool,
        key_length_state: int,
        key_length_min: int,
        default_key_state: int,
        default_key_list: List[str],
        default_key_reversed: bool,
        complexity_state: int,
        complexity_regex: List[str],
        complexity_repeated: bool,
):
    if key_length_min > 0:
        yield from _check_key_length(key, message, key_length_state, report_only, key_length_min)
    if default_key_list:
        yield from _check_default_key(
            key, message, default_key_state, report_only, default_key_list, default_key_reversed
        )
    if complexity_regex:
        yield from _check_key_complexity(key, message, complexity_state, report_only, complexity_regex)
    if complexity_repeated:
        yield from _check_repeated_chars(key, message, complexity_state, report_only)


def parse_snmp_configuration(string_table: StringTable):
    return string_table


def discovery_snmp_configuration(section) -> DiscoveryResult:
    yield Service()


def check_snmp_configuration(params, section) -> CheckResult:
    config_cache = get_config_cache()
    snmp_host = False
    report_only = params['report_only']

    try:
        # test for CMK 2.2.x
        snmp_host = config_cache.is_snmp_host(host_name())
        snmp_credentials = config_cache._snmp_credentials(host_name())
        snmp_backend = config_cache.get_snmp_backend(host_name()).value
        snmp_version1 = config_cache._is_host_snmp_v1(host_name())
    except AttributeError:
        # try cmk 2.0.x - 2.1.x
        host_config = config_cache.get_host_config(host_name())
        snmp_config = host_config.snmp_config(None)
        snmp_credentials = snmp_config.credentials
        snmp_backend = snmp_config.snmp_backend.value
        snmp_version1 = host_config._is_host_snmp_v1()
        if snmp_credentials:
            snmp_host = True

    if snmp_host:
        snmp_version = '1' if snmp_version1 else '2c' if type(snmp_credentials) is str else '3'

        excepted, state = params['snmp_version']
        yield from _check_expected(snmp_version, 'Version', state, report_only, excepted)

        excepted, state = params['snmp_backend']
        yield from _check_expected(snmp_backend.lower(), 'Backend', state, report_only, excepted)

        if snmp_version == '3':
            excepted, state = params['v3_level']
            yield from _check_expected(snmp_credentials[0].lower(), 'Level', state, report_only, excepted)

            if len(snmp_credentials) > 2:
                excepted, state = params['v3_authentication']
                excepted = list(map(str.upper, excepted))
                yield from _check_expected(snmp_credentials[1].upper(), 'Authentication', state, report_only, excepted)

            try:
                snmp_encryption = snmp_credentials[4]
            except IndexError:
                pass
            else:
                excepted, state = params['v3_encryption']
                excepted = list(map(str.upper, excepted))
                yield from _check_expected(snmp_encryption.upper(), 'Encryption', state, report_only, excepted)

        min_key_length, state_min_key = params['min_key_length']
        default_keys, reversed_keys, state_default_key = params['default_keys']
        default_keys = list(map(str.lower, default_keys))
        key_complexity, repeated_chars, state_key_complexity = params['key_complexity']

        if snmp_version != '3':
            message = 'Community string'
            yield from _check_key(
                key=snmp_credentials,
                message=message,
                report_only=report_only,
                key_length_state=state_min_key,
                key_length_min=min_key_length,
                default_key_state=state_default_key,
                default_key_list=default_keys,
                default_key_reversed=reversed_keys,
                complexity_state=state_key_complexity,
                complexity_regex=key_complexity,
                complexity_repeated=repeated_chars,
            )
        else:
            if len(snmp_credentials) > 2:
                message = 'Authentication key'
                yield from _check_key(
                    key=snmp_credentials[3],
                    message=message,
                    report_only=report_only,
                    key_length_state=state_min_key,
                    key_length_min=min_key_length,
                    default_key_state=state_default_key,
                    default_key_list=default_keys,
                    default_key_reversed=reversed_keys,
                    complexity_state=state_key_complexity,
                    complexity_regex=key_complexity,
                    complexity_repeated=repeated_chars,
                )
            if len(snmp_credentials) == 6:
                message = 'Encryption key'
                yield from _check_key(
                    key=snmp_credentials[5],
                    message=message,
                    report_only=report_only,
                    key_length_state=state_min_key,
                    key_length_min=min_key_length,
                    default_key_state=state_default_key,
                    default_key_list=default_keys,
                    default_key_reversed=reversed_keys,
                    complexity_state=state_key_complexity,
                    complexity_regex=key_complexity,
                    complexity_repeated=repeated_chars,
                )
                if snmp_credentials[3] == snmp_credentials[5]:
                    yield from _result(
                        message=f'Authentication and Encryption key are identical',
                        state=params['auth_enc_key_identical'],
                        report_only=report_only)
                else:
                    yield Result(state=State.OK, notice=F'Authentication and Encryption key are different')

    else:
        yield Result(state=State.OK, summary='No SNMP host')


register.snmp_section(
    name='snmp_configuration',
    parse_function=parse_snmp_configuration,
    fetch=SNMPTree(
        base='.1.3.6.1.2.1.1',  # 
        oids=[
            '1',  # sysDescr
        ]),
    detect=exists('.1.3.6.1.2.1.1.1.0', ),  # sysDescr
)

register.check_plugin(
    name='snmp_configuration',
    service_name='SNMP Configuration',
    discovery_function=discovery_snmp_configuration,
    check_function=check_snmp_configuration,
    check_default_parameters={
        'report_only': False,
        'snmp_version': (['3', 1]),
        'v3_level': (['authpriv'], 1),
        'v3_authentication': (['sha', 'sha-224', 'sha-256', 'sha-384', 'sha-512'], 1),
        'v3_encryption': (['aes'], 1),
        'snmp_backend': (['inline', 'classic'], 1),
        'default_keys': (['public', 'private'], True, 1),
        'min_key_length': (10, 1),
        'auth_enc_key_identical': 1,
        'key_complexity': (['(?=.*\\d)', '(?=.*[a-z])', '(?=.*[A-Z])', '(?=.*\\W)', '(?=.{10,})'], True, 1),
    },
    check_ruleset_name='snmp_configuration',
)
