#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2

# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-07-09
# File  : snmp_configuration.py
#
# WATO file for snmp_configuration check plugin
#
# 2023-07-09: initial release
# 2023-08-04: renamed from snmp_version to snmp_configuration
#             added options for report only, reversed keys, no repeated chars
#             optimized layout
#             moved rule set from Networking to Applications, Processes & Services


from cmk.gui.i18n import _
from cmk.gui.valuespec import (
    Dictionary,
    MonitoringState,
    Tuple,
    ListChoice,
    Integer,
    ListOfStrings,
    FixedValue,
    Checkbox,
)

from cmk.gui.plugins.wato.utils import (
    CheckParameterRulespecWithoutItem,
    rulespec_registry,
    RulespecGroupCheckParametersApplications,
)


def _parameter_valuespec_snmp_configuration():
    return Dictionary(
        elements=[
            ('report_only',
             FixedValue(
                 value=True,
                 title=_('Report only'),
                 totext=_(''),
                 help=_('If enabled the check will not create any alerts (WAR/CRIT)'),
             )),
            ('snmp_version',
             Tuple(
                 title=_('SNMP Version'),
                 elements=[
                     ListChoice(
                         choices=[
                             ('1', 'Version 1'),
                             ('2c', 'Version 2c'),
                             ('3', 'Version 3')
                         ],
                         allow_empty=False,
                         help=_('Select excepted SNMP versions. Default is "Version 3".'),
                         default_value=['3'],
                     ),
                     MonitoringState(
                         label=_('Monitoring state if version not in list'),
                         default_value=1,
                     )
                 ])),
            ('v3_level',
             Tuple(
                 title=_('SNMP v3 level'),
                 elements=[
                     ListChoice(
                         choices=[
                             ('authpriv', 'authentication and privacy'),
                             ('authnopriv', 'authentication and no privacy'),
                             ('noauthnopriv', 'no authentication and no privacy'),
                         ],
                         allow_empty=False,
                         help=_('Select excepted SNMP v3 protocol level. Default is "authentication and privacy".'),
                         default_value=['authpriv'],
                     ),
                     MonitoringState(
                         label=_('Monitoring state if level is not list'),
                         default_value=1,
                     )
                 ])),
            ('v3_authentication',
             Tuple(
                 title=_('SNMP v3 authentication'),
                 elements=[
                     ListChoice(
                         choices=[
                             ('md5', 'MD5'),
                             ('sha', 'SHA'),
                             ('sha-224', 'SHA-224'),
                             ('sha-256', 'SHA-256'),
                             ('sha-384', 'SHA-384'),
                             ('sha-512', 'SHA-512'),
                         ],
                         allow_empty=False,
                         help=_('Select excepted SNMP v3 authentication. Default is "SHA*".'),
                         default_value=['sha', 'sha-224', 'sha-256', 'sha-384', 'sha-512'],
                     ),
                     MonitoringState(
                         label=_('Monitoring state if authentication is not in list'),
                         default_value=1,
                     )
                 ])),
            ('v3_encryption',
             Tuple(
                 title=_('SNMP v3 encryption'),
                 elements=[
                     ListChoice(
                         choices=[
                             ('des', 'DES'),
                             ('aes', 'AES-128'),
                         ],
                         allow_empty=False,
                         help=_('Select excepted encryption. Default is "AES-128".'),
                         default_value=['aes'],
                     ),
                     MonitoringState(
                         label=_('Monitoring state if encryption is not in list'),
                         default_value=1,
                     )
                 ])),
            ('snmp_backend',
             Tuple(
                 title=_('SNMP backend'),
                 elements=[
                     ListChoice(
                         choices=[
                             ('inline', 'Inline'),
                             ('classic', 'Classic'),
                             ('storedwalk', 'Stored walk')
                         ],
                         allow_empty=False,
                         help=_('Select excepted SNMP backend. Default is "Inline/Classic".'),
                         default_value=['inline', 'classic'],
                     ),
                     MonitoringState(
                         label=_('Monitoring state if backend not in list'),
                         default_value=1,
                     )
                 ])),
            ('min_key_length',
             Tuple(
                 title=_('Minimal key length'),
                 elements=[
                     Integer(
                         minvalue=0,
                         help=_(
                             'Minimal expected Community string/key length. '
                             'Use "0" to disable this check. Default is "10".'
                         ),
                         default_value=10,
                     ),
                     MonitoringState(
                         label=_('Monitoring state if key length below minimal key length'),
                         help=_(
                             'Set the monitoring state when the length of the Community string (SNMP v1/2c) '
                             'or the authentication/encryption key (SNMP v3) is below min. length. Default is WARN'
                         ),
                         default_value=1,
                     )
                 ])),
            ('auth_enc_key_identical',
             MonitoringState(
                 title=_('Monitoring state when authentication and encryption keys are identical'),
                 help=_(
                     'Set the monitoring state when authentication key and encryption key are identical. '
                     'This setting is for SNMP v3 only. Default is WARN.'
                 ),
                 default_value=1,
             )),
            ('default_keys',
             Tuple(
                 title=_('Check default key values'),
                 elements=[
                     ListOfStrings(
                         help=_(
                             'List of default Community strings/keys to check against. Default is "public, private".'
                         ),
                         default_value=['public', 'private'],
                     ),
                     Checkbox(
                         label=_('Check reversed keys'),
                         help=_('This will also check for keys in reverse. So "cilbup" will match "public".'),
                         default_value=True,
                     ),
                     MonitoringState(
                         label=_('Monitoring state if key used is a default key'),
                         help=_(
                             'Set the monitoring state when the key used is in the list of default keys. '
                             'Default is WARN'
                         ),
                         default_value=1,
                     )
                 ])),
            ('key_complexity',
             Tuple(
                 title=_('Key complexity'),
                 elements=[
                     ListChoice(
                         choices=[
                             ('(?=.*\\d)', 'Digit'),
                             ('(?=.*[a-z])', 'Lowercase'),
                             ('(?=.*[A-Z])', 'Uppercase'),
                             ('(?=.*\\W)', 'Special')
                         ],
                         help=_(
                             'Select expected character types in the Community string or authentication/encryption key.'
                             ' Default is "Number, Lower case, Upper case, Special".'),
                         default_value=['(?=.*\\d)', '(?=.*[a-z])', '(?=.*[A-Z])', '(?=.*\\W)'],
                     ),
                     Checkbox(
                         label=_('no repeated chars'),
                         help=_('This will also check if the key contains repeated characters..'),
                         default_value=True,
                     ),
                     MonitoringState(
                         label=_('Monitoring state if key complexity not met'),
                         default_value=1,
                     )
                 ])),
        ])


rulespec_registry.register(
    CheckParameterRulespecWithoutItem(
        check_group_name='snmp_configuration',
        group=RulespecGroupCheckParametersApplications,
        match_type='dict',
        parameter_valuespec=_parameter_valuespec_snmp_configuration,
        title=lambda: _('Checkmk SNMP Configuration'),
    ))
